import json

def print_registries():
    registries = get_registries()
        
    for key in registries.keys():
        print("--->", key.replace("minecraft:", ""))

def get_registries():
    with open('../functions/mc_data/registries.json') as f:
        registries = json.load(f)
        
    return registries

def get_registry_keys(registry: str):
    registries = get_registries()
    
    if not registry.startswith("minecraft:"):
        registry = "minecraft:" + registry
        
    return registries[registry]

def print_registry_keys(registry: str):
    keys = get_registry_keys(registry)
    
    for key in keys:
        print("--->", key.replace("minecraft:", ""))
        
def get_commands():
    with open('../functions/mc_data/commands.json') as f:
        commands = json.load(f)
    return commands

def print_commands():
    commands = get_commands()     
    for command in commands["children"]:
        print("--->", command)

computed_commands = ["title", "bossbar", "setblock", "fill", "summon", "say",
                     "ban", "ban-ip", "banlist", "kick", "op", "deop", "seed",
                     "difficulty", "weather"]
excluded_commands = ["advancement"]
complex_commands = []

def print_todo_commands():
    commands = get_commands()     
    for command in commands["children"]:
        if command not in computed_commands and command not in excluded_commands:
            print("--->", command)
            
def print_done_commands():
    for command in computed_commands:
        print("--->", command)
        
def print_excluded_commands():
    for command in excluded_commands:
        print("--->", command)
        
def print_complex_commands():
    for command in complex_commands:
        print("--->", command)

def print_commands_summary():
    print("Done:")
    print_done_commands()
    print()
    print("Todo:")
    print_todo_commands()
    print()
    print("Complex:")
    print_complex_commands()
    print()
    print("Excluded:")
    print_excluded_commands()
         
print_commands_summary()


from typing import Union
from shulker.functions.base_functions import *
import re

############ SAY ############

def meta_say(text: str) -> dict:
    instructions = {"list": []}
    instructions["list"].append(f"say {text}")
    return instructions

def say(text: str) -> Union[bool, str]:
    """This function sends a message in the chat"""
    check_output_channel()
    
    instructions = meta_say(text)
    
    for line in instructions["list"]:
        status = post(line)

######## BAN & KICK ########

def meta_ban(target: str, reason: str) -> dict:
    instructions = {"list": []}
    if reason is None:
        reason = ""
    instructions["list"].append(f"ban {target} {reason}")
    return instructions

def meta_banlist(option: str) -> dict:
    instructions = {"list": []}
    instructions["list"].append(f"banlist {option}")
    return instructions

def ban(target: str, reason: Union[str, None]) -> Union[bool, str]:
    """This function bans a player"""
    check_output_channel()
    
    instructions = meta_ban(target, reason)
    
    for line in instructions["list"]:
        status = post(line)
 
def ban_ip(target: str, reason: str) -> Union[bool, str]:
    """This function bans an IP"""
    check_output_channel()
    
    instructions = meta_ban(target, reason)
    
    for line in instructions["list"]:
        status = post(line)
        
def banlist(option: str) -> Union[bool, str]:
    """This functions fetches the banlist"""
    check_output_channel()
    
    instructions = meta_banlist(option)
    
    for line in instructions["list"]:
        status = post(line)

def meta_kick(target: str, reason: str) -> dict:
    instructions = {"list": []}
    if reason is None:
        reason = ""
    instructions["list"].append(f"kick {target} {reason}")
    return instructions

def kick(target: str, reason: Union[str, None]) -> Union[bool, str]:
    """This function kicks a player"""
    check_output_channel()
    
    instructions = meta_kick(target, reason)
    
    for line in instructions["list"]:
        status = post(line)

############ OP ############

def meta_op(target: str) -> dict:
    instructions = {"list": []}
    instructions["list"].append(f"op {target}")
    return instructions

def meta_deop(target: str) -> dict:
    instructions = {"list": []}
    instructions["list"].append(f"deop {target}")
    return instructions

def op(target: str) -> Union[bool, str]:
    """This function gives an operator status to a player"""
    check_output_channel()
    
    instructions = meta_op(target)
    
    for line in instructions["list"]:
        status = post(line)
        
def deop(target: str) -> Union[bool, str]:
    """This function removes an operator status from a player"""
    check_output_channel()
    
    instructions = meta_deop(target)
    
    for line in instructions["list"]:
        status = post(line)
        
############# SEED #############

def meta_seed() -> dict:
    instructions = {"list": []}
    instructions["list"].append(f"seed")
    return instructions

def seed() -> Union[bool, str]:
    """This function fetches the seed of the world"""
    check_output_channel()
    
    instructions = meta_seed()
    
    for line in instructions["list"]:
        status = post(line)
    
    seed = re.match(r"Seed: \[(-{0,1}(\d+))\]", status)
    return seed.group(1)

############# DIFFICULTY #############

def meta_set_difficulty(option: str) -> dict:
    instructions = {"list": []}
    instructions["list"].append(f"difficulty {option}")
    return instructions

def meta_get_difficulty() -> dict:
    instructions = {"list": []}
    instructions["list"].append(f"difficulty")
    return instructions

def set_difficulty(option: str) -> Union[bool, str]:
    """This function sets the difficulty of the world"""
    check_output_channel()
    
    instructions = meta_set_difficulty(option)
    
    for line in instructions["list"]:
        status = post(line)
        
def get_difficulty() -> Union[bool, str]:
    """This function fetches the difficulty of the world"""
    check_output_channel()
    
    instructions = meta_get_difficulty()
    
    for line in instructions["list"]:
        status = post(line)
    
    difficulty = re.match(r"The difficulty is (\w+)", status)
    return difficulty.group(1)


########### WEATHER ###########

def meta_weather(option: str) -> dict:
    instructions = {"list": []}
    instructions["list"].append(f"weather {option}")
    return instructions

def weather(option: str) -> Union[bool, str]:
    """This function sets the weather of the world
    Options: clear, rain, thunder"""
    
    check_output_channel()
    
    instructions = meta_weather(option)
    
    for line in instructions["list"]:
        status = post(line)


import json
import os

from typing import Union

from shulker.components.BlockCoordinates import BlockCoordinates
from shulker.components.NBT import NBT
from shulker.components.Coordinates import Coordinates

from shulker.functions.base_functions import *


def meta_summon(entity: str, coords: BlockCoordinates, nbt_data: NBT) -> dict:
    instructions = {"list": []}
    instructions["list"].append(f"summon {entity} {coords} {nbt_data}")
    return instructions


def summon(
    entity: str,
    coords: Union[BlockCoordinates, Coordinates, tuple],
    nbt_data: Union[NBT, dict, str, None] = None
) -> Union[bool, str]:
    """
    Returns a bool that is set to True
    if no message was sent back by the game or the
    message itself if there was an issue

    Summons an entity
    """

    check_output_channel()
    
    entities = entity_list()
        
    if type(entity) is not str:
        raise TypeError(f"Expected type str, got {type(entity)}")
    elif entity.replace("minecraft:", "") not in entities:
        raise ValueError(f"Entity {entity} is not a valid entity")
    
    if nbt_data is not None:
        nbt_data = format_arg(nbt_data, NBT)
    else:
        nbt_data = ""

    coords = format_arg(coords, Coordinates)
        
    instructions = meta_summon(entity, coords, nbt_data)

    for line in instructions["list"]:
        status = post(line)
        print(line)
        
    return True if status.startswith("") else status



from .base_functions import nest_commands, entity_list
from .get_block import get_block, meta_get_block
from .set_block import set_block, meta_set_block
from .set_image import set_image, meta_set_image, print_palette
from .set_text import set_text, meta_set_text
from .set_zone import set_zone, meta_set_zone
from .summon import summon, meta_summon
from .get_player_nbt import get_player_nbt, get_player_pos, meta_get_player_nbt

from .set_gui import create_bossbar, meta_create_bossbar
from .set_gui import add_bossbar, meta_add_bossbar
from .set_gui import list_bossbar, meta_list_bossbar
from .set_gui import remove_bossbar, meta_remove_bossbar
from .set_gui import get_bossbar, meta_get_bossbar
from .set_gui import set_bossbar, meta_set_bossbar
from .set_gui import show_gui, meta_show_gui
from .set_gui import clear_gui, meta_clear_gui

from .miscellaneous import say, meta_say
from .miscellaneous import ban, ban_ip, meta_ban, banlist, meta_banlist, kick, meta_kick
from .miscellaneous import op, deop, meta_op, meta_deop
from .miscellaneous import seed, meta_seed
from .miscellaneous import set_difficulty, meta_set_difficulty, get_difficulty, meta_get_difficulty
from .miscellaneous import weather, meta_weather


from setuptools import setup, find_packages

# install requires should pull the list from requirements.txt
with open('requirements.txt') as f:
    requirements = f.read().splitlines()

setup(
    name='shulker',
    version='0.3.11',
    author='PortalHubYT',
    author_email='portalhub.business@gmail.com',
    description='A minecraft interface using RCON',
    url='https://github.com/PortalHubYT/mc_api',
    packages=find_packages(),
    package_data={
        'shulker': ['functions/*.json', 'components/*.json', 'server/*.json']
    },
    # install requires should pull the list from requirements.txt
    install_requires=requirements
)


from nbtlib import parse_nbt
from nbtlib import Byte, Short, Int, Long, Float, Double, String, ByteArray, IntArray, LongArray, List, Compound
from typing import Union

class NBT:
    def __init__(self, compound: Union[dict, str, Compound] = "{}"):
        
        if compound in [None, "", "{}", {}]:
            nbt = None
        elif type(compound) is Compound:
            nbt = compound
        elif type(compound) is dict:
            nbt = parse_nbt(str(compound))
        elif isinstance(compound, str):
            nbt = parse_nbt(compound)
        else:
            raise ValueError(f"Expected dict, str or Compound, got {type(compound)}")
        
        if nbt:
            for key in nbt:
                object.__setattr__(self, key, nbt[key])
            
    def flatten(self, arg):
        if type(arg) is Byte:
            return f'{int(arg)}b'
        elif type(arg) is Short:
            return f'{int(arg)}s'
        elif type(arg) is Int:
            return f'{int(arg)}'
        elif type(arg) is Long:
            return f'{int(arg)}L'
        elif type(arg) is Float:
            return f'{float(arg)}f'
        elif type(arg) is Double:
            return f'{float(arg)}d'
        elif type(arg) is String: # refer to test_NBT_with_args in test_NBT.py
            if '"' in arg:
                return f"'{str(arg)}'"
            else:
                return f'"{str(arg)}"'
        elif type(arg) is ByteArray:
            return f'[{",".join([int(str(int(x))) for x in arg])}]b'
        elif type(arg) is IntArray:
            return f'[{",".join([str(int(x)) for x in arg])}]'
        elif type(arg) is LongArray:
            return f'[{",".join([str(int(x)) for x in arg])}]L'
        elif isinstance(arg, List):
            return f'[{",".join([self.flatten(x) for x in arg])}]'
        elif type(arg) is Compound:
            if arg == {}:
                return "{}"
            return str(NBT(arg))

    def __setattr__(self, name, value):
        value = parse_nbt(str(value))
        super().__setattr__(name, value)
        
    def __str__(self):

        buff = ""

        for key in dir(self):
            if key.startswith("__"):
                continue
            elif key == "flatten":
                continue

            value = getattr(self, key)
            value = self.flatten(value)

            buff += f"{key}:{value},"

        if buff != "":
            buff = buff[:-1]
            return f"{{{buff}}}"
        else:
            return ""
